require 'redis/hash_key'
require 'forwardable'
require 'securerandom'
require 'singularity/password_hash'

module Singularity
  class Profile
    extend Forwardable

    attr_reader :prefix, :identity, :redis

    def initialize(identity, options = {})
      raise ArgumentError, "identity must be present" unless identity.to_s.length > 0

      @prefix       = options.fetch(:prefix) { Singularity.profile_prefix }
      @identity     = identity
      @redis        = options.fetch(:redis) { Redis.current }
    end

    def_delegators :profile, :[], :[]=
    def_delegators :profile, :all, :bulk_get, :bulk_set, :each, :empty?, :fetch
    def_delegators :profile, :has_key?, :include?, :incr, :incrby, :key, :key?
    def_delegators :profile, :keys, :member?, :vals, :values, :clear, :delete

    def password_hash
      self['password_hash']
    end

    def password_hash=(value)
      self['password_hash'] = value
    end

    def password_salt
      self['password_salt']
    end

    def password_salt=(value)
      self['password_salt'] = value
    end

    def password_iterations
      iters = self['password_iterations'].to_i
      iters = PasswordHash.iterations unless iters > 0

      iters
    end

    def password_iterations=(value)
      self['password_iterations'] = value
    end

    def password?(value)
      return false unless password_hash.to_s.length > 0
      return false unless password_salt.to_s.length > 0

      PasswordHash.new(password_hash).matches?(value, password_salt, iterations: password_iterations)
    end

  private

    def profile
      @profile ||= Redis::HashKey.new("#{ prefix }:#{ identity }", marshal: true).tap do |hash|
        hash.instance_variable_set(:@redis, redis)
      end
    end
  end
end
